%6/9/2014: Updated codes to use a local /tmp folder when unix is used.
%5/7/2014: Updated codes to use the GAMLSS parameter functions
%ed8: parallel random seeds, saving rand_seeds and loads
%ed7-02: target quantile is revised using 45000+45000=90000 sample.
%ed7-01: target quantile is revised using 45000 sample.
%ed7: rng(sum(i*100*clock))inside of parfor to avoid repetition (Meeting_112712 slide)
%ed6: rng shuffle & Random seed entered from the kernel & Log file
%ed5: Spline
%ed4: Linear interpolation of parameters of GEV distribution
%ed3: Cubic smoothing spline of parameters of GEV distribution (instead of MLE coefficients of paramter equation)
%ed2: For around 50% CPU usage (three simulators)
%ed1_parallel: Parallel running code is added to calculate standard error
%POE_ed1: IS for estimating POE given a target quantile & Use NREL_simulator_1
%Ed5 : GEV, weighted mean of shape param, cubic location, quadratic scale, MLE
%Ed4 : Type_of_load specified in front
%Ed3 : Metamodel - Skip MLE stage in the parameter estimation
%Ed2 : Metamodel - Weighted least squares for mean estimation
global p_var;
global kappa;
global delta;
global mu;
global sigma; %In order to set it independent of x when performing experiment for delta
global mu_hat;
global sigma_hat;
global N_T;
p_var = 1;
delta=1;
kappa=1;
mu = (@(x) (0.95.*delta).*x.^2.*(1 + 0.5.*cos(10.*kappa.*x) + 0.5.*cos(20.*kappa.*x)));
sigma = (@(x) 1 + 0.7.*abs(x) + 0.4.*cos(x) + 0.3.*cos(14.*x));
mu_hat = (@(x) (0.95.*delta).*x.^2.*(1 + 0.5.*cos(10.*kappa.*x) + 0.5.*cos(20.*kappa.*x)));
sigma_hat = (@(x) 1 + 0.7.*abs(x) + 0.4.*cos(x) + 0.3.*cos(14.*x));

x_mu = zeros(1,p_var); %input variable
x_sigma = eye(p_var);  %input variable

%% Parameters to adjust at every PBS submission

%Load type and target quantile
LoadType = 1; %k=1, Blade 1 root flapwise bending moment ; k=2, Blade 1 root edgewise bending moment
%target_quantile = 1.5; %8.589013e+003; %8.587490e+003; %9.288587e+003; %8.600400e+003; %9.601406e+003; %       %Corresponding to P_T = 0.05 ---- 8/16/12 meeting slide p.6
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%parameter settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Target POE
alpha=0.01;
target_probability=1-alpha;
%%% \tilde{s}_k=(1-det/k^beta)s+det/k^beta(1-s)
det = 0.1;
beta = 0.5;
%%% parameter a in RM and gamma in PR
a=10;
gamma =-0.5;
% Simulation size
%%% N_T
N_total = 100; %1000; % N = Total computational resource (i.e., total number of sampling execution from the conditional distribution, Y | X = x_i.)
%%% m
N_long = 30; %round(N_total / N_short_ave); % the sample size from the optimal-allocation IS distribution (i.e., how many v_i are sampled in the o-a IS estimator)
N_T=N_total;

%%% initial theta1
theta_iter=3;

% Parallel running
%%% Number of parallel cores/ Number of replications
n_parallel = 2; 
%%% Number of interations K
n_repetition = 5;  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
randseed = 10;
CMC_rep=10^6;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);

l_1=theta_iter;
max_q = theta_iter;

% Simulator folder number to be used (from 1 through 4): line 196
simNum = 2;  %Simulator number is not significant and can be overlapped if unix is used due to 6/9/2014 migration to /tmp.

% Counter (to avoid the same output filename; no other important purpose is intended; Form: 1th, 2th, 3th, 4th, ... NOT 1st, 2nd, 3rd, 4th,...)
countNum = 1;

%% Setup
isUnix = 0; %1;     %1 if Unix; 0 if Windows

POE_est_sample_size = n_parallel*n_repetition;   % !!! It should be an integer multiplication of n_parallel.

% Reseed unpredictably the random number generator : http://blogs.mathworks.com/loren/2011/07/07/simpler-control-of-random-number-generation-in-matlab/
rng shuffle
    
    
%% Save in .mat and log management
if LoadType == 1    %flapwise
        LoadTypeStr = 'Out_tip'; 
elseif LoadType ==2 %edgewise     
        LoadTypeStr = 'In_tip';
end 

Save_log_filename = horzcat('SIS1_',LoadTypeStr,num2str(theta_iter),'_',num2str(N_total),'_',num2str(N_long),'_',num2str(n_parallel),'_',num2str(n_repetition),'rep','_','sim',num2str(simNum),'_ct',num2str(countNum)); %'Edge9300_3000_1500_20_1rep_sim1';    

        if isUnix ==1
            IO_folder = '/scratch/ebyon_flux/yjchoe/'; %'/home/yjchoe/Workspace/IO/';
        elseif isUnix ==0
            IO_folder = 'C:\Users\qiyun\Desktop\tipmetamodel\';    
        end

        savefile = horzcat(IO_folder, Save_log_filename, '.mat');
        logfile = horzcat(IO_folder, Save_log_filename, '.txt');
        File_log = fopen(logfile,'a+');        


Time_Start = tic;        

%% P_meta_given_x

f_meta = (@(x) normpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
sx = (@(x) 1-normcdf(theta_iter, mu(x), sigma(x)));
index_repetition=1;
s_meta = (@(x) (1-det/index_repetition^beta).*sx(x)+det/index_repetition^beta.*(1-sx(x)));
kernel_meta = (@(x) f_meta(x).* sqrt(s_meta(x)./N_T + (1-1/N_T).*s_meta(x).^2));  %kernel is C_q * q_sis1
inside_of_sqrt = (@(x)s_meta(x)/N_total + (1-1/N_total).*(s_meta(x)).^2);
%%%%%%%%%% diff(f_meta(x),x):-(2251799813685248*x*exp(-x^2/2))/5644425081792261=0.3989*x*exp(-x^2/2)
approx_df =(@(x) 0.3989.*x.*exp(-x.^2/2).*s_meta(x)-f_meta(x).^2);
norm_constant_IS_pdf = quadgk(kernel_meta,-Inf,Inf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Start of parfor: loop for parallel running

if isUnix ==1
    sched= findResource('scheduler', 'type', 'mpiexec'); 
    set(sched, 'MpiexecFileName', '/home/software/rhel6/mpiexec/bin/mpiexec') 
    set(sched, 'EnvironmentSetMethod', 'setenv') 
    matlabpool(sched, n_parallel)   
elseif isUnix ==0 
    parpool(n_parallel);   
end

spmd
s = RandStream.create('mrg32k3a','NumStreams',numlabs,'StreamIndices',labindex, 'Seed', 'shuffle');
RandStream.setGlobalStream(s);
end

POE_estimates = repmat(vpa(0.0),[n_repetition, n_parallel]); %POE_estimates(index_repetition, index_parfor)  %zeros(1,12);
POE_estimates_add = repmat(vpa(0.0),[n_repetition, n_parallel]);
Random_seeds = zeros([n_repetition, n_parallel, N_long, N_total]);
Y_in = zeros([n_repetition, n_parallel, N_long, N_total]); 
Y_out = zeros([n_repetition, n_parallel, N_long, N_total]);
all_sample_x = zeros([n_repetition, n_parallel, N_long]);
all_q_x = zeros([n_repetition, n_parallel, N_long]);
%1st dimension: repetition, 2nd dim: parallel, 3rd dim: i in N_long, 4th dim: j in N_short_allocation (max value <= N_total)
Time_started_for_sampling = tic;
x = cell(1,n_parallel);
y = cell(1,n_parallel);
f = cell(1,n_parallel);
r = cell(1,n_parallel);
h = cell(1,n_parallel);
h1 = cell(1,n_parallel);
h2 = cell(1,n_parallel);
sort_y =  cell(1,n_parallel);
sort_y1 =  cell(1,n_parallel);
sort_y2 =  cell(1,n_parallel);
G =  cell(1,n_parallel);
Gk =  cell(1,n_parallel);
s =  cell(1,n_parallel);
S =  cell(1,n_parallel);
q_est=cell(1,n_parallel);
z_est=cell(1,n_parallel);
l_RM=cell(1,n_parallel);
theta=cell(1,n_parallel);
N_i = cell(1,n_parallel);
n = zeros(1,n_parallel);
Theta=zeros(1,n_repetition+1);
Theta(1)=theta_iter;
N_alloc=[];
POE=[];
%mean_q_est=zeros([1,n_repetition]);
%index_parfor = 1; 
flag=0;
C1=cell(1,n_parallel);
c1=zeros(1,n_parallel);
for index_repetition = 1:n_repetition    
for index_parfor = 1:n_parallel
%% Step 4: Sample x from the IS distribution (rejection method )   
     C1{index_parfor}=[C1{index_parfor};c1(index_parfor)];
     sample_x = Kernel_based_IS_rnd(c1(index_parfor), x_sigma, kernel_meta, norm_constant_IS_pdf, N_long);
     ind_centr=find(abs(approx_df(sample_x))==min(abs(approx_df(sample_x))));
     c1(index_parfor)=sample_x(ind_centr);

    %sample_x = Kernel_based_IS_rnd(x_mu, x_sigma, kernel_meta, norm_constant_IS_pdf, N_long);

    all_sample_x(index_repetition, index_parfor,:) = sample_x;
    %Histogram comparison of samples (from original pdf vs. IS pdf): see GEV_meta_IS_pdf_rnd.m     


%% Step 5: Calculate the short-term sample allocation


    old_digits = digits;   % For Variable-precision accuracy
    digits(50);
    %digits(old_digits); %Recover default accuracy: 32bit

    % Determine Optimal allocation of short-term sample size (Note: rounding error)
    allocation_proportion_for_obs_x = repmat(vpa(0.0),[1,N_long+1]);
    for index_i = 1:N_long  %drange()
        P = sx(sample_x(index_i));
        allocation_proportion_for_obs_x(index_i) = vpa(sqrt( (1-P)     / ( 1+(N_total-1)*P )    ) );   %proportional constant for the observed x_i =(f/q)*sqrt(P(Y>y_T | X=x)*(1-P(Y>y_T | X=x))) = sqrt( (1-P)     / ( 1+(N_total-1)*P )    )      : 'Meeting_080212.pptx' p.14.
    end
    alloc_normalizing_constant = vpa(0.0);
    for index_i = 1:N_long  %drange(1:N_long)
        alloc_normalizing_constant = vpa(alloc_normalizing_constant + allocation_proportion_for_obs_x(index_i));    
    end


    % Sort the proportions for x_i so that the most important (higest proportion) x_i has the first index.
[sorted_proportion, original_order]=sort(allocation_proportion_for_obs_x, 'descend');
temp_N_short_allocation = zeros(1, N_long); 
adjusted_normalizing_constant = alloc_normalizing_constant;
for index = N_long:-1:2 %drange(N_long:-1:2)
    remained_sample_size = N_total-sum(temp_N_short_allocation(index+1:N_long));   %N - [  sum_j=(k+1)^(N_long) N_j ]
    adjusted_normalizing_constant =  vpa(adjusted_normalizing_constant - sorted_proportion(index+1));
    unrounded_alloc_for_obs_x = vpa(remained_sample_size * sorted_proportion(index)/adjusted_normalizing_constant);
    if (double(unrounded_alloc_for_obs_x)<1)
        temp_N_short_allocation(index) = 1;
    else    %(double(unrounded_alloc_for_obs_x)>=1)
        temp_N_short_allocation(index) = round(double(unrounded_alloc_for_obs_x));
    end       
end 
temp_N_short_allocation(1) = N_total - sum(temp_N_short_allocation(2:N_long));    %N_total - [  sum_j=2^(N_long) N_j ]


N_short_allocation  = zeros(1, N_long);   % N_i : the allocated sample size for each stratum i
% Order to recover the original indice
for index = 1:N_long    %drange(1:N_long)
    N_short_allocation(original_order(index)) = temp_N_short_allocation(index);
end
    N_alloc=[N_alloc;N_short_allocation];

    
    %digits(old_digits); %Recover default accuracy: 32bit

%% !! Step 6 (Long computational time): Sample Y for each x

    %old_digits = digits;   % For Variable-precision accuracy
    %digits(50);
    

    temp_rand_seeds = zeros([N_long,N_total]);

    Y_i_j_matrix = zeros([N_long,max(N_short_allocation)]); %Output j-th observation of Y for X = x_i
    N_load_type = 2; % Number of load types that will be obtained from each simulation run
    k_type_of_load_for_a_run_i_j = zeros([N_long,N_total, N_load_type]); % To contain every loading output from each simulation run indexed as (i,j): e.g. k=1, Blade 1 root flapwise bending moment ; k=2, Blade 1 root edgewise bending moment
    for index_i = 1:N_long  %drange(1:N_long)   % index i for each x_i
            for index_j = 1:N_short_allocation(index_i) %drange(1:N_short_allocation(index_i))  % index j for Y_j^(i)
                k_type_of_load_for_a_run_i_j(index_i,index_j,:) = [normrnd(mu(sample_x(index_i)),sigma(sample_x(index_i))),normrnd(0,1)];
                Y_i_j_matrix(index_i,index_j)= k_type_of_load_for_a_run_i_j(index_i,index_j,LoadType);
            end
    end
    Random_seeds(index_repetition, index_parfor, :, :) = temp_rand_seeds;
    Y_in(index_repetition, index_parfor,:,:) = k_type_of_load_for_a_run_i_j(:,:,1);
    Y_out(index_repetition, index_parfor,:,:) = k_type_of_load_for_a_run_i_j(:,:,2);
     
 
    digits(old_digits); %Recover default accuracy: 32bit
    
        
    
    



%% Step 8: Deliver the POE estimate for l_T (=target_quantile)

%load('C:\Research\Implementation\CMC_IO\IS_NREL_metamodel_N_T_1000_long_50.mat');

    Ind_Y_i_j_matrix = zeros([N_long,max(N_short_allocation)]); %Output Ind_Y_ij = I(Y_ij > y_T) for j-th observation of Y for X = x_i
    sum_for_short_term = zeros([1,N_long]);    %\sum_(j=1)^(N_i) I(Y_ij > y_T)
    term_for_short_term = repmat(vpa(0.0),[1,N_long]);   %f(x_i)/q(x_i) * (1/N_i) * \sum_(j=1)^(N_i) I(Y_ij > y_T)
    likelihood_at_x = repmat(vpa(0.0),[1,N_long]);  %this is technically likelihood divided by norm_constant_IS_pdf =(f/q)*(1/C_q) for a computational reason, C_q is multiplied in the last stage.
    for index_i = 1:N_long  %drange(1:N_long)   % index i for each x_i
            for index_j = 1:N_short_allocation(index_i) %drange(1:N_short_allocation(index_i))  % index j for Y_j^(i) 
                Ind_Y_i_j_matrix(index_i,index_j)= (double(vpa(Y_i_j_matrix(index_i,index_j)-theta_iter))>0);  %Y_T, the target quantile, is given,
                sum_for_short_term(index_i) = Ind_Y_i_j_matrix(index_i,index_j)+ sum_for_short_term(index_i);
            end
            likelihood_at_x(index_i) = vpa(1/sqrt(inside_of_sqrt(sample_x(index_i)))); %f(x_i)/q(x_i)*(1/C_q) = 
            term_for_short_term(index_i) = vpa(likelihood_at_x(index_i) / N_short_allocation(index_i) * sum_for_short_term(index_i));          
    end
     
     
    sum_for_long_term = vpa(0.0);
    for index_i = 1:N_long  %drange(1:N_long)  
       sum_for_long_term = vpa(sum_for_long_term + term_for_short_term(index_i));
    end
    
      POE_estimate_for_l_T = vpa(sum_for_long_term / N_long * norm_constant_IS_pdf);     %Note: (1/C_q)* N_long = expectation( sum_of_likelihood )  
      POE_estimates(index_repetition, index_parfor) = POE_estimate_for_l_T;
      POE =[POE,POE_estimate_for_l_T];
      fprintf('%gs elapsed: Additional %g sampling is done from the parallel lab %d (/%d).\n',toc(Time_started_for_sampling), N_total, index_parfor,n_parallel);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  adaptive      
      t_quantile = zeros(1,n_parallel);
      for index_parfor = 1:n_parallel
      N_i{index_parfor} = N_alloc(index_parfor,:);
      x{index_parfor} = all_sample_x(1:index_repetition, index_parfor,1:N_long);
%     f{index_parfor} = raylpdf(x{index_parfor},Rayleigh_scale_par);
      f{index_parfor} = normpdf(x{index_parfor},0,1);
      r{index_parfor} = norm_constant_IS_pdf/sqrt(inside_of_sqrt(x{index_parfor}));
      r{index_parfor}(isinf(r{index_parfor}))=0;
      y{index_parfor}=Y_in(1:index_repetition, index_parfor,1:N_long,1:N_total);
      h{index_parfor}=y{index_parfor}(y{index_parfor}>=max_q);     
      sort_y{index_parfor}=sort(h{index_parfor});        
      n(index_parfor)=length(sort_y{index_parfor});
      G{index_parfor}=zeros(1,n(index_parfor));
%     s{index_parfor} = zeros(index_repetition,N_long);
%     S{index_parfor} = zeros(1,index_repetition);
      for i=1:n(index_parfor)
          l=sort_y{index_parfor}(i);
          for k=1:index_repetition      
              for j=1:N_long
              s{index_parfor}(k,j) = sum(logical(y{index_parfor}(k,1,j,:)>l))/N_i{index_parfor}(j)*r{index_parfor}(k,1,j);   
              end
              S{index_parfor}(k) = sum(s{index_parfor}(k,:))/N_long;
          end
          G{index_parfor}(i) = sum(S{index_parfor})/index_repetition;
      end  
      if min(G{index_parfor})>alpha
          q_est{index_parfor}(index_repetition)=max(sort_y{index_parfor});
      else 
          q_est{index_parfor}(index_repetition)= sort_y{index_parfor}(min(find(logical(G{index_parfor}<alpha))));
      end
%      for i=1:n(index_parfor)
%          l=sort_y{index_parfor}(i);
          for k=1:index_repetition  
              lk=Theta(index_repetition);
              for j=1:N_long
              s{index_parfor}(k,j) = sum(logical(y{index_parfor}(k,1,j,:)>lk))/N_i{index_parfor}(j)*r{index_parfor}(k,1,j);   
              end
              S{index_parfor}(k) = sum(s{index_parfor}(k,:))/N_long;
          end
          Gk{index_parfor}(index_repetition) = sum(S{index_parfor})/index_repetition;
%      end    
      if index_repetition==1
      l_RM{index_parfor}(index_repetition)=lk+a*(index_repetition^gamma)*(Gk{index_parfor}(index_repetition)-alpha);
      else
      l_RM{index_parfor}(index_repetition)=l_RM{index_parfor}(index_repetition-1)+a*(index_repetition^gamma)*(Gk{index_parfor}(index_repetition)-alpha);
      l_RM{index_parfor}(index_repetition)=mean(l_RM{index_parfor}(1:index_repetition));
      end
      
      if max(G{index_parfor})>(index_repetition+1)/index_repetition*alpha
      z_est{index_parfor}(index_repetition)=sort_y{index_parfor}(max(find(logical(G{index_parfor}>(index_repetition+1)/index_repetition*alpha))));
      if  q_est{index_parfor}(index_repetition)>l_RM{index_parfor}(index_repetition)
          theta{index_parfor}(index_repetition)= max(l_RM{index_parfor}(index_repetition),z_est{index_parfor}(index_repetition));
      else 
          theta{index_parfor}(index_repetition)=z_est{index_parfor}(index_repetition);
      end  
          temp_theta(index_parfor)=theta{index_parfor}(index_repetition);     
      else    
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
      %empty set
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      Y_in_add = zeros([1, n_parallel, N_long*(n_repetition-index_repetition), N_total*(n_repetition-index_repetition)]); 
      Y_out_add = zeros([1, n_parallel, N_long*(n_repetition-index_repetition), N_total*(n_repetition-index_repetition)]);
      Random_seeds_add = zeros([1, n_parallel, N_long*(n_repetition-index_repetition), N_total*(n_repetition-index_repetition)]);      
      
      k_stop=index_repetition;
      for i=1:k_stop
      if Gk{index_parfor}>=(i+n_repetition-k_stop)/i*alpha    
      theta_iter=Theta(i);
      end
      end
      k_choose=find(Theta==theta_iter);
      f_meta = (@(x) normpdf(x,x_mu,x_sigma));   %NOTE: only works for p_var = 1.  %multidimensional version: (@(x) mvnpdf(x,x_mu,x_sigma));
      sx = (@(x) 1-normcdf(theta_iter, mu(x), sigma(x)));
      s_meta = (@(x) 1-normcdf(theta_iter, mu_hat(x), sigma_hat(x)));   

      kernel_meta = (@(x) f_meta(x).* sqrt(s_meta(x)./N_T + (1-1/N_T).*s_meta(x).^2));  %kernel is C_q * q_sis1
      inside_of_sqrt = (@(x)s_meta(x)/N_total + (1-1/N_total).*(s_meta(x)).^2);
      norm_constant_IS_pdf = quadgk(kernel_meta,-Inf,Inf);
      
      sample_x_add = Kernel_based_IS_rnd(x_mu, x_sigma, kernel_meta, norm_constant_IS_pdf, N_long*(n_repetition-index_repetition));
      old_digits = digits;   % For Variable-precision accuracy
      digits(50);
      %digits(old_digits); %Recover default accuracy: 32bit

      % Determine Optimal allocation of short-term sample size (Note: rounding error)
      allocation_proportion_for_obs_x = repmat(vpa(0.0),[1,N_long*(n_repetition-index_repetition)+1]);
      for index_i = 1:(N_long*(n_repetition-index_repetition))  %drange()
 %      P = GEV_P_given_x(sample_x(index_i));
        P = sx(sample_x_add(index_i));
        allocation_proportion_for_obs_x(index_i) = vpa(sqrt( (1-P)/ ( 1+(N_total*(n_repetition-index_repetition)-1)*P )    ) );   %proportional constant for the observed x_i =(f/q)*sqrt(P(Y>y_T | X=x)*(1-P(Y>y_T | X=x))) = sqrt( (1-P)     / ( 1+(N_total-1)*P )    )      : 'Meeting_080212.pptx' p.14.
      end
      alloc_normalizing_constant = vpa(0.0);
      for index_i = 1:(N_long*(n_repetition-index_repetition))  %drange(1:N_long)
        alloc_normalizing_constant = vpa(alloc_normalizing_constant + allocation_proportion_for_obs_x(index_i));    
      end
      [sorted_proportion, original_order]=sort(allocation_proportion_for_obs_x, 'descend');
      temp_N_short_allocation = zeros(1, (N_long*(n_repetition-index_repetition))); 
      adjusted_normalizing_constant = alloc_normalizing_constant;
      for index = (N_long*(n_repetition-index_repetition)):-1:2 %drange(N_long:-1:2)
      remained_sample_size = (N_total*(n_repetition-index_repetition)-1)-sum(temp_N_short_allocation(index+1:(N_long*(n_repetition-index_repetition))));   %N - [  sum_j=(k+1)^(N_long) N_j ]
      adjusted_normalizing_constant =  vpa(adjusted_normalizing_constant - sorted_proportion(index+1));
      unrounded_alloc_for_obs_x = vpa(remained_sample_size * sorted_proportion(index)/adjusted_normalizing_constant);
      if (double(unrounded_alloc_for_obs_x)<1)
        temp_N_short_allocation(index) = 1;
      else    %(double(unrounded_alloc_for_obs_x)>=1)
        temp_N_short_allocation(index) = round(double(unrounded_alloc_for_obs_x));
      end       
end 
temp_N_short_allocation(1) = (N_total*(n_repetition-index_repetition)) - sum(temp_N_short_allocation(2:(N_long*(n_repetition-index_repetition))));    %N_total - [  sum_j=2^(N_long) N_j ]


N_short_allocation  = zeros(1, (N_long*(n_repetition-index_repetition)));   % N_i : the allocated sample size for each stratum i
for index = 1:(N_long*(n_repetition-index_repetition))    %drange(1:N_long)
    N_short_allocation(original_order(index)) = temp_N_short_allocation(index);
end
  % N_alloc=[N_alloc;N_short_allocation];    
    temp_rand_seeds = zeros([(N_long*(n_repetition-index_repetition)),(N_total*(n_repetition-index_repetition))]);
    Y_i_j_matrix_add = zeros([(N_long*(n_repetition-index_repetition)),max(N_short_allocation)]); %Output j-th observation of Y for X = x_i
    N_load_type = 2; % Number of load types that will be obtained from each simulation run
    k_type_of_load_for_a_run_i_j_add = zeros([N_long*(n_repetition-index_repetition),(N_total*(n_repetition-index_repetition)), N_load_type]); % To contain every loading output from each simulation run indexed as (i,j): e.g. k=1, Blade 1 root flapwise bending moment ; k=2, Blade 1 root edgewise bending moment
    for index_i = 1:(N_long*(n_repetition-index_repetition))  %drange(1:N_long)   % index i for each x_i
            for index_j = 1:N_short_allocation(index_i) %drange(1:N_short_allocation(index_i))  % index j for Y_j^(i)
                k_type_of_load_for_a_run_i_j_add(index_i,index_j,:) = [normrnd(mu(sample_x_add(index_i)),sigma(sample_x_add(index_i))),normrnd(0,1)];
                Y_i_j_matrix_add(index_i,index_j)= k_type_of_load_for_a_run_i_j_add(index_i,index_j,LoadType);
            end
    end
    Random_seeds_add(1, index_parfor, :, :) = temp_rand_seeds;
    Y_in_add(1, index_parfor,:,:) = k_type_of_load_for_a_run_i_j_add(:,:,1);
    Y_out_add(1, index_parfor,:,:) = k_type_of_load_for_a_run_i_j_add(:,:,2); 
    digits(old_digits); %Recover default accuracy: 32bit
    
    Ind_Y_i_j_matrix = zeros([N_long*(n_repetition-index_repetition),max(N_short_allocation)]); %Output Ind_Y_ij = I(Y_ij > y_T) for j-th observation of Y for X = x_i
    sum_for_short_term = zeros([1,N_long*(n_repetition-index_repetition)]);    %\sum_(j=1)^(N_i) I(Y_ij > y_T)
    term_for_short_term = repmat(vpa(0.0),[1,N_long*(n_repetition-index_repetition)]);   %f(x_i)/q(x_i) * (1/N_i) * \sum_(j=1)^(N_i) I(Y_ij > y_T)
    likelihood_at_x = repmat(vpa(0.0),[1,N_long*(n_repetition-index_repetition)]);  %this is technically likelihood divided by norm_constant_IS_pdf =(f/q)*(1/C_q) for a computational reason, C_q is multiplied in the last stage.
    
    y_add{index_parfor}=Y_in_add(1, index_parfor,1:N_long*(n_repetition-index_repetition),1:N_total*(n_repetition-index_repetition));
    h1{index_parfor}=y{index_parfor}(y{index_parfor}>=theta_iter); 
    h2{index_parfor}=y_add{index_parfor}(y_add{index_parfor}>=theta_iter);
      sort_y1{index_parfor}=sort(h1{index_parfor});
      sort_y2{index_parfor}=sort(h2{index_parfor});
      sort_y{index_parfor}=[sort_y1{index_parfor};sort_y2{index_parfor}];
      n(index_parfor)=length(sort_y{index_parfor});
      G{index_parfor}=zeros(1,n(index_parfor));
   
      for i=1:n(index_parfor)
          l=sort_y{index_parfor}(i);
          for k=1:index_repetition      
              for j=1:N_long
              s{index_parfor}(k,j) = sum(logical(y{index_parfor}(k,1,j,:)>l))/N_i{index_parfor}(j)*r{index_parfor}(k,1,j);   
              end
              S{index_parfor}(k) = sum(s{index_parfor}(k,:))/N_long;
          end
          G{index_parfor}(i) = sum(S{index_parfor})/index_repetition;      
   
          for index_i = 1: (N_long*(n_repetition-index_repetition))  %drange(1:N_long)   % index i for each x_i
            for index_j = 1:N_short_allocation(index_i) %drange(1:N_short_allocation(index_i))  % index j for Y_j^(i) 
                Ind_Y_i_j_matrix(index_i,index_j)= (double(vpa(Y_i_j_matrix_add(index_i,index_j)-l))>0);  %Y_T, the target quantile, is given,
                sum_for_short_term(index_i) = Ind_Y_i_j_matrix(index_i,index_j)+ sum_for_short_term(index_i);
            end
            likelihood_at_x(index_i) = vpa(1/sqrt(inside_of_sqrt(sample_x_add(index_i)))); %f(x_i)/q(x_i)*(1/C_q) = 
            term_for_short_term(index_i) = vpa(likelihood_at_x(index_i) / N_short_allocation(index_i) * sum_for_short_term(index_i));          
          end
          sum_for_long_term = vpa(0.0);
          for index_i = 1:(N_long*(n_repetition-index_repetition))  %drange(1:N_long)  
          sum_for_long_term = vpa(sum_for_long_term + term_for_short_term(index_i));
          end
          POE_estimate_for_l_T_add = vpa(sum_for_long_term /(N_long*(n_repetition-index_repetition)) * norm_constant_IS_pdf);
          POE_estimates_add(index_repetition, index_parfor) = POE_estimate_for_l_T_add;
          
          G{index_parfor}(i) = k_choose/(k_choose+n_repetition-k_stop)*G{index_parfor}(i)+(n_repetition-k_stop)/(k_choose+n_repetition-k_stop)*POE_estimate_for_l_T_add;
      end
      
      if min(G{index_parfor})>alpha
          q_est{index_parfor}(index_repetition)=max(sort_y{index_parfor});
      else 
          q_est{index_parfor}(index_repetition)= sort_y{index_parfor}(min(find(logical(G{index_parfor}<alpha))));
      end     
      %Note: (1/C_q)* N_long = expectation( sum_of_likelihood )  
      
   %  POE =[POE,POE_estimate_for_l_T_add];
      fprintf('%gs elapsed: Additional %g sampling is done from the parallel lab %d (/%d).\n',toc(Time_started_for_sampling), N_total, index_parfor,n_parallel);     
      
      flag=1;
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      end
      end
      if flag == 1
        break;
      end
      theta_iter=min(temp_theta);
 
      Theta(index_repetition+1)=theta_iter;
      max_q=max(Theta);
      sx = (@(x) 1-normcdf(theta_iter, mu(x), sigma(x)));
      s_meta = (@(x) (1-det/index_repetition^beta).*sx(x)+det/index_repetition^beta.*(1-sx(x)));

      kernel_meta = (@(x) f_meta(x).* sqrt(s_meta(x)./N_T + (1-1/N_T).*s_meta(x).^2));  %kernel is C_q * q_sis1
      inside_of_sqrt = (@(x)s_meta(x)/N_total + (1-1/N_total).*(s_meta(x)).^2);
      norm_constant_IS_pdf = quadgk(kernel_meta,-Inf,Inf);
               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% End of parfor: loop for parallel running
     
fprintf(File_log, 'The loop %g out of %g is finished. \r\n', index_repetition, POE_est_sample_size/n_parallel); 

 dsave(savefile);
end


delete(gcp)

fclose(File_log);

Time_Elapsed = toc(Time_Start)      %#ok<NOPTS>

dsave(savefile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% print results
randseed = 10;
CMC_rep=10^6;
target_quantile = CMC_quantile(target_probability, CMC_rep, randseed, mu, sigma, x_mu, x_sigma);
q1=zeros([1,n_parallel]);
theta_q2=zeros([1,n_parallel]);
for i=1:n_parallel
q1(i)=q_est{i}(n_repetition);
theta_q2(i)=theta{i}(n_repetition);
end
fprintf('Empirical Bias ASIS1: %g \n', double(mean(q1)-target_quantile));
fprintf('Sample std ASIS1: %g \n', double(sqrt(var(q1))));
fprintf('Empirical Bias ASIS2: %g \n', double(mean(theta_q2)-target_quantile));
fprintf('Sample std ASIS2: %g \n', double(sqrt(var(theta_q2))));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Saving data
cd('C:\Users\qiyun\Desktop\ASIS1dim\Est')
filename=sprintf('ASIS_10Neg%d_a%d_gamma%d.mat',abs(log10(alpha)),a,-gamma*10);
save(filename,'theta','l_RM','z_est','q_est')
cd('C:\Users\qiyun\Desktop\ASIS1dim\Samples')
inputfilename=sprintf('X_ASIS_10Neg%d_a%d_gamma%d.txt',abs(log10(alpha)),a,-gamma*10);
outputfilename=sprintf('Y_ASIS_10Neg%d_a%d_gamma%d.txt',abs(log10(alpha)),a,-gamma*10);
fid=fopen(inputfilename,'w');
fprintf(fid, '%.4f \n', all_sample_x);
fclose(fid);
fid=fopen(outputfilename,'w');
fprintf(fid, '%.4f \n', Y_in);
fclose(fid);
cd('C:\Users\qiyun\Desktop\ASIS1dim')

